
From: Ashok Raj <ashok.raj@intel.com>

The use of non-shortcut version of routines breaking CPU hotplug.  The option
to select this via cmdline also is deleted with the physflat patch, hence
directly placing this code under CONFIG_HOTPLUG_CPU.

We dont want to use broadcast mode IPI's when hotplug is enabled.  This causes
bad effects in send IPI to a cpu that is offline which can trip when the cpu
is in the process of being kicked alive.

Signed-off-by: Ashok Raj <ashok.raj@intel.com>
Cc: Andi Kleen <ak@muc.de>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 arch/x86_64/kernel/genapic_flat.c |   10 ++++++++++
 1 files changed, 10 insertions(+)

diff -puN arch/x86_64/kernel/genapic_flat.c~x86_64dont-do-broadcast-ipis-when-hotplug-is-enabled-in-flat-mode arch/x86_64/kernel/genapic_flat.c
--- devel/arch/x86_64/kernel/genapic_flat.c~x86_64dont-do-broadcast-ipis-when-hotplug-is-enabled-in-flat-mode	2005-08-04 10:42:08.000000000 -0700
+++ devel-akpm/arch/x86_64/kernel/genapic_flat.c	2005-08-04 10:42:08.000000000 -0700
@@ -78,8 +78,18 @@ static void flat_send_IPI_mask(cpumask_t
 
 static void flat_send_IPI_allbutself(int vector)
 {
+#ifndef CONFIG_HOTPLUG_CPU
 	if (((num_online_cpus()) - 1) >= 1)
 		__send_IPI_shortcut(APIC_DEST_ALLBUT, vector,APIC_DEST_LOGICAL);
+#else
+	cpumask_t allbutme = cpu_online_map;
+	int me = get_cpu(); /* Ensure we are not preempted when we clear */
+	cpu_clear(me, allbutme);
+
+	if (!cpus_empty(allbutme))
+		flat_send_IPI_mask(allbutme, vector);
+	put_cpu();
+#endif
 }
 
 static void flat_send_IPI_all(int vector)
_
